@@ -9,7 +9,6 @@ from typing import (
     TypeVar,
     Generic,
     Union,
-    Iterator,
     Iterable,
 )
 
@@ -43,12 +42,12 @@ class _Read1(Awaitable):
 class _ReadUntil(Awaitable):
     __slots__ = ["sep", "max_bytes"]
 
-    def __init__(self, sep, max_bytes=None):
+    def __init__(self, sep: str, max_bytes: int | None = None) -> None:
         self.sep = sep
         self.max_bytes = max_bytes
 
 
-class PeekBuffer(Awaitable):
+class _PeekBuffer(Awaitable):
     __slots__: list[str] = []
 
 
@@ -62,7 +61,7 @@ class Parser(Generic[T]):
     read = _Read
     read1 = _Read1
     read_until = _ReadUntil
-    peek_buffer = PeekBuffer
+    peek_buffer = _PeekBuffer
 
     def __init__(self) -> None:
         self._buffer = io.StringIO()
@@ -104,14 +103,14 @@ class Parser(Generic[T]):
         while tokens:
             yield popleft()
 
-        while pos < data_size or isinstance(self._awaiting, PeekBuffer):
+        while pos < data_size or isinstance(self._awaiting, _PeekBuffer):
 
             _awaiting = self._awaiting
             if isinstance(_awaiting, _Read1):
                 self._awaiting = self._gen.send(data[pos : pos + 1])
                 pos += 1
 
-            elif isinstance(_awaiting, PeekBuffer):
+            elif isinstance(_awaiting, _PeekBuffer):
                 self._awaiting = self._gen.send(data[pos:])
 
             elif isinstance(_awaiting, _Read):
@@ -156,8 +155,7 @@ class Parser(Generic[T]):
                 yield popleft()
 
     def parse(self, on_token: Callable[[T], None]) -> Generator[Awaitable, str, None]:
-        return
-        yield
+        yield from ()
 
 
 if __name__ == "__main__":
@@ -167,7 +165,6 @@ if __name__ == "__main__":
         def parse(
             self, on_token: Callable[[str], None]
         ) -> Generator[Awaitable, str, None]:
-            data = yield self.read1()
             while True:
                 data = yield self.read1()
                 if not data:
