@@ -1,15 +1,11 @@
 from __future__ import annotations
 
-import sys
 from dataclasses import dataclass
 from typing import Iterable, MutableMapping
 
 import rich.repr
 
-if sys.version_info >= (3, 10):
-    from typing import TypeAlias
-else:  # pragma: no cover
-    from typing_extensions import TypeAlias
+from textual._typing import TypeAlias
 
 BindingType: TypeAlias = "Binding | tuple[str, str, str]"
 
@@ -25,17 +21,17 @@ class NoBinding(Exception):
 @dataclass(frozen=True)
 class Binding:
     key: str
-    """Key to bind. This can also be a comma-separated list of keys to map multiple keys to a single action."""
+    """str: Key to bind. This can also be a comma-separated list of keys to map multiple keys to a single action."""
     action: str
-    """Action to bind to."""
+    """str: Action to bind to."""
     description: str
-    """Description of action."""
+    """str: Description of action."""
     show: bool = True
-    """Show the action in Footer, or False to hide."""
+    """bool: Show the action in Footer, or False to hide."""
     key_display: str | None = None
-    """How the key should be shown in footer."""
+    """str | None: How the key should be shown in footer."""
     universal: bool = False
-    """Allow forwarding from app to focused widget."""
+    """bool: Allow forwarding from app to focused widget."""
 
 
 @rich.repr.auto
@@ -111,6 +107,16 @@ class Bindings:
         key_display: str | None = None,
         universal: bool = False,
     ) -> None:
+        """Bind keys to an action.
+
+        Args:
+            keys (str): The keys to bind. Can be a comma-separated list of keys.
+            action (str): The action to bind the keys to.
+            description (str, optional): An optional description for the binding.
+            show (bool, optional): A flag to say if the binding should appear in the footer.
+            key_display (str | None, optional): Optional string to display in the footer for the key.
+            universal (bool, optional): Allow forwarding from the app to the focused widget.
+        """
         all_keys = [key.strip() for key in keys.split(",")]
         for key in all_keys:
             self.keys[key] = Binding(
