@@ -17,9 +17,13 @@ a method which evaluates the query, such as first() and last().
 from __future__ import annotations
 
 from typing import cast, Generic, TYPE_CHECKING, Iterator, TypeVar, overload
+import asyncio
 
 import rich.repr
 
+from .. import events
+from .._context import active_app
+from ..await_remove import AwaitRemove
 from .errors import DeclarationError, TokenError
 from .match import match
 from .model import SelectorSet
@@ -42,6 +46,10 @@ class NoMatches(QueryError):
     """No nodes matched the query."""
 
 
+class TooManyMatches(QueryError):
+    """Too many nodes matched the query."""
+
+
 class WrongType(QueryError):
     """Query result was not of the correct type."""
 
@@ -208,6 +216,49 @@ class DOMQuery(Generic[QueryType]):
         else:
             raise NoMatches(f"No nodes match {self!r}")
 
+    @overload
+    def only_one(self) -> Widget:
+        ...
+
+    @overload
+    def only_one(self, expect_type: type[ExpectType]) -> ExpectType:
+        ...
+
+    def only_one(
+        self, expect_type: type[ExpectType] | None = None
+    ) -> Widget | ExpectType:
+        """Get the *only* matching node.
+
+        Args:
+            expect_type (type[ExpectType] | None, optional): Require matched node is of this type,
+                or None for any type. Defaults to None.
+
+        Raises:
+            WrongType: If the wrong type was found.
+            TooManyMatches: If there is more than one matching node in the query.
+
+        Returns:
+            Widget | ExpectType: The matching Widget.
+        """
+        # Call on first to get the first item. Here we'll use all of the
+        # testing and checking it provides.
+        the_one = self.first(expect_type) if expect_type is not None else self.first()
+        try:
+            # Now see if we can access a subsequent item in the nodes. There
+            # should *not* be anything there, so we *should* get an
+            # IndexError. We *could* have just checked the length of the
+            # query, but the idea here is to do the check as cheaply as
+            # possible.
+            _ = self.nodes[1]
+            raise TooManyMatches(
+                "Call to only_one resulted in more than one matched node"
+            )
+        except IndexError:
+            # The IndexError was got, that's a good thing in this case. So
+            # we return what we found.
+            pass
+        return the_one
+
     @overload
     def last(self) -> Widget:
         ...
@@ -232,16 +283,14 @@ class DOMQuery(Generic[QueryType]):
         Returns:
             Widget | ExpectType: The matching Widget.
         """
-        if self.nodes:
-            last = self.nodes[-1]
-            if expect_type is not None:
-                if not isinstance(last, expect_type):
-                    raise WrongType(
-                        f"Query value is wrong type; expected {expect_type}, got {type(last)}"
-                    )
-            return last
-        else:
+        if not self.nodes:
             raise NoMatches(f"No nodes match {self!r}")
+        last = self.nodes[-1]
+        if expect_type is not None and not isinstance(last, expect_type):
+            raise WrongType(
+                f"Query value is wrong type; expected {expect_type}, got {type(last)}"
+            )
+        return last
 
     @overload
     def results(self) -> Iterator[Widget]:
@@ -301,11 +350,15 @@ class DOMQuery(Generic[QueryType]):
             node.toggle_class(*class_names)
         return self
 
-    def remove(self) -> DOMQuery[QueryType]:
-        """Remove matched nodes from the DOM"""
-        for node in self:
-            node.remove()
-        return self
+    def remove(self) -> AwaitRemove:
+        """Remove matched nodes from the DOM.
+
+        Returns:
+            AwaitRemove: An awaitable object that waits for the widgets to be removed.
+        """
+        app = active_app.get()
+        await_remove = app._remove_nodes(list(self))
+        return await_remove
 
     def set_styles(
         self, css: str | None = None, **update_styles
