@@ -14,34 +14,48 @@ from threading import Event, Thread
 if TYPE_CHECKING:
     from rich.console import Console
 
-from .. import log
+import rich.repr
 
-from .. import events
+from .. import log
 from ..driver import Driver
 from ..geometry import Size
 from .._types import MessageTarget
 from .._xterm_parser import XTermParser
 from .._profile import timer
+from .. import events
 
 
+@rich.repr.auto
 class LinuxDriver(Driver):
     """Powers display and input for Linux / MacOS"""
 
-    def __init__(self, console: "Console", target: "MessageTarget") -> None:
-        super().__init__(console, target)
+    def __init__(
+        self,
+        console: "Console",
+        target: "MessageTarget",
+        *,
+        debug: bool = False,
+        size: tuple[int, int] | None = None,
+    ) -> None:
+        super().__init__(console, target, debug=debug, size=size)
         self.fileno = sys.stdin.fileno()
         self.attrs_before: list[Any] | None = None
         self.exit_event = Event()
         self._key_thread: Thread | None = None
 
+    def __rich_repr__(self) -> rich.repr.Result:
+        yield "debug", self._debug
+
     def _get_terminal_size(self) -> tuple[int, int]:
         width: int | None = 80
         height: int | None = 25
+        import shutil
+
         try:
-            width, height = os.get_terminal_size(sys.__stdin__.fileno())
+            width, height = shutil.get_terminal_size()
         except (AttributeError, ValueError, OSError):
             try:
-                width, height = os.get_terminal_size(sys.__stdout__.fileno())
+                width, height = shutil.get_terminal_size()
             except (AttributeError, ValueError, OSError):
                 pass
         width = width or 80
@@ -61,6 +75,14 @@ class LinuxDriver(Driver):
         # Note: E.g. lxterminal understands 1000h, but not the urxvt or sgr
         #       extensions.
 
+    def _enable_bracketed_paste(self) -> None:
+        """Enable bracketed paste mode."""
+        self.console.file.write("\x1b[?2004h")
+
+    def _disable_bracketed_paste(self) -> None:
+        """Disable bracketed paste mode."""
+        self.console.file.write("\x1b[?2004l")
+
     def _disable_mouse_support(self) -> None:
         write = self.console.file.write
         write("\x1b[?1000l")  #
@@ -71,18 +93,21 @@ class LinuxDriver(Driver):
 
     def start_application_mode(self):
 
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
 
-        def on_terminal_resize(signum, stack) -> None:
+        def send_size_event():
             terminal_size = self._get_terminal_size()
             width, height = terminal_size
-            event = events.Resize(self._target, Size(width, height))
-            self.console.size = terminal_size
+            textual_size = Size(width, height)
+            event = events.Resize(self._target, textual_size, textual_size)
             asyncio.run_coroutine_threadsafe(
                 self._target.post_message(event),
                 loop=loop,
             )
 
+        def on_terminal_resize(signum, stack) -> None:
+            send_size_event()
+
         signal.signal(signal.SIGWINCH, on_terminal_resize)
 
         self.console.set_alt_screen(True)
@@ -114,15 +139,15 @@ class LinuxDriver(Driver):
         self.console.show_cursor(False)
         self.console.file.write("\033[?1003h\n")
         self.console.file.flush()
-        self._key_thread = Thread(
-            target=self.run_input_thread, args=(asyncio.get_event_loop(),)
-        )
-        width, height = self.console.size = self._get_terminal_size()
-        asyncio.run_coroutine_threadsafe(
-            self._target.post_message(events.Resize(self._target, Size(width, height))),
-            loop=loop,
-        )
+        self._key_thread = Thread(target=self.run_input_thread, args=(loop,))
+        send_size_event()
         self._key_thread.start()
+        self._request_terminal_sync_mode_support()
+        self._enable_bracketed_paste()
+
+    def _request_terminal_sync_mode_support(self):
+        self.console.file.write("\033[?2026$p")
+        self.console.file.flush()
 
     @classmethod
     def _patch_lflag(cls, attrs: int) -> int:
@@ -148,15 +173,16 @@ class LinuxDriver(Driver):
             if not self.exit_event.is_set():
                 signal.signal(signal.SIGWINCH, signal.SIG_DFL)
                 self._disable_mouse_support()
-                termios.tcflush(self.fileno, termios.TCIFLUSH)
                 self.exit_event.set()
                 if self._key_thread is not None:
                     self._key_thread.join()
+                termios.tcflush(self.fileno, termios.TCIFLUSH)
         except Exception as error:
             # TODO: log this
             pass
 
     def stop_application_mode(self) -> None:
+        self._disable_bracketed_paste()
         self.disable_input()
 
         if self.attrs_before is not None:
@@ -189,38 +215,24 @@ class LinuxDriver(Driver):
                     return True
             return False
 
-        parser = XTermParser(self._target, more_data)
+        parser = XTermParser(self._target, more_data, self._debug)
+        feed = parser.feed
 
         utf8_decoder = getincrementaldecoder("utf-8")().decode
         decode = utf8_decoder
         read = os.read
+        EVENT_READ = selectors.EVENT_READ
 
         try:
             while not self.exit_event.is_set():
                 selector_events = selector.select(0.1)
                 for _selector_key, mask in selector_events:
-                    if mask | selectors.EVENT_READ:
+                    if mask | EVENT_READ:
                         unicode_data = decode(read(fileno, 1024))
-                        for event in parser.feed(unicode_data):
+                        for event in feed(unicode_data):
                             self.process_event(event)
         except Exception as error:
             log(error)
         finally:
             with timer("selector.close"):
                 selector.close()
-
-
-if __name__ == "__main__":
-    from time import sleep
-    from rich.console import Console
-    from .. import events
-
-    console = Console()
-
-    from ..app import App
-
-    class MyApp(App):
-        async def on_mount(self, event: events.Mount) -> None:
-            self.set_timer(5, callback=self.close_messages)
-
-    MyApp.run()
