@@ -17,8 +17,15 @@ if TYPE_CHECKING:
 class WindowsDriver(Driver):
     """Powers display and input for Windows."""
 
-    def __init__(self, console: "Console", target: "MessageTarget") -> None:
-        super().__init__(console, target)
+    def __init__(
+        self,
+        console: "Console",
+        target: "MessageTarget",
+        *,
+        debug: bool = False,
+        size: tuple[int, int] | None = None,
+    ) -> None:
+        super().__init__(console, target, debug=debug, size=size)
         self.in_fileno = sys.stdin.fileno()
         self.out_fileno = sys.stdout.fileno()
 
@@ -42,9 +49,17 @@ class WindowsDriver(Driver):
         write("\x1b[?1006l")
         self.console.file.flush()
 
+    def _enable_bracketed_paste(self) -> None:
+        """Enable bracketed paste mode."""
+        self.console.file.write("\x1b[?2004h")
+
+    def _disable_bracketed_paste(self) -> None:
+        """Disable bracketed paste mode."""
+        self.console.file.write("\x1b[?2004l")
+
     def start_application_mode(self) -> None:
 
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
 
         self._restore_console = win32.enable_application_mode()
 
@@ -52,6 +67,7 @@ class WindowsDriver(Driver):
         self._enable_mouse_support()
         self.console.show_cursor(False)
         self.console.file.write("\033[?1003h\n")
+        self._enable_bracketed_paste()
 
         app = active_app.get()
 
@@ -73,6 +89,7 @@ class WindowsDriver(Driver):
             pass
 
     def stop_application_mode(self) -> None:
+        self._disable_bracketed_paste()
         self.disable_input()
         if self._restore_console:
             self._restore_console()
