@@ -16,6 +16,7 @@ if TYPE_CHECKING:
     from .timer import Timer as TimerClass
     from .timer import TimerCallback
     from .widget import Widget
+    import asyncio
 
 
 @rich.repr.auto
@@ -23,8 +24,7 @@ class Event(Message):
     """The base class for all events."""
 
     def __rich_repr__(self) -> rich.repr.Result:
-        return
-        yield
+        yield from ()
 
 
 @rich.repr.auto
@@ -127,14 +127,6 @@ class Unmount(Mount, bubble=False, verbose=False):
     """Sent when a widget is unmounted and may not longer receive messages."""
 
 
-class Remove(Event, bubble=False):
-    """Sent to a widget to ask it to remove itself from the DOM."""
-
-    def __init__(self, sender: MessageTarget, widget: Widget) -> None:
-        self.widget = widget
-        super().__init__(sender)
-
-
 class Show(Event, bubble=False):
     """Sent when a widget has become visible."""
 
