@@ -251,7 +251,7 @@ class MessagePump(metaclass=MessagePumpMeta):
         self._timers.add(timer)
         return timer
 
-    def call_later(self, callback: Callable, *args, **kwargs) -> None:
+    def call_after_refresh(self, callback: Callable, *args, **kwargs) -> None:
         """Schedule a callback to run after all messages are processed and the screen
         has been refreshed. Positional and keyword arguments are passed to the callable.
 
@@ -263,6 +263,16 @@ class MessagePump(metaclass=MessagePumpMeta):
         message = messages.InvokeLater(self, partial(callback, *args, **kwargs))
         self.post_message_no_wait(message)
 
+    def call_later(self, callback: Callable, *args, **kwargs) -> None:
+        """Schedule a callback to run after all messages are processed in this object.
+        Positional and keywords arguments are passed to the callable.
+
+        Args:
+            callback (Callable): Callable to call next.
+        """
+        message = events.Callback(self, callback=partial(callback, *args, **kwargs))
+        self.post_message_no_wait(message)
+
     def _on_invoke_later(self, message: messages.InvokeLater) -> None:
         # Forward InvokeLater message to the Screen
         self.app.screen._invoke_later(message.callback)
@@ -274,7 +284,7 @@ class MessagePump(metaclass=MessagePumpMeta):
     async def _on_close_messages(self, message: messages.CloseMessages) -> None:
         await self._close_messages()
 
-    async def _close_messages(self) -> None:
+    async def _close_messages(self, wait: bool = True) -> None:
         """Close message queue, and optionally wait for queue to finish processing."""
         if self._closed or self._closing:
             return
@@ -284,8 +294,9 @@ class MessagePump(metaclass=MessagePumpMeta):
             await timer.stop()
         self._timers.clear()
         await self._message_queue.put(events.Unmount(sender=self))
+        Reactive._reset_object(self)
         await self._message_queue.put(None)
-        if self._task is not None and asyncio.current_task() != self._task:
+        if wait and self._task is not None and asyncio.current_task() != self._task:
             # Ensure everything is closed before returning
             await self._task
 
