@@ -1,14 +1,8 @@
 from __future__ import annotations
 
-import sys
 from functools import partial
 from typing import cast
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal  # pragma: no cover
-
 import rich.repr
 from rich.console import RenderableType
 from rich.text import Text, TextType
@@ -18,6 +12,7 @@ from ..css._error_tools import friendly_list
 from ..message import Message
 from ..reactive import Reactive
 from ..widgets import Static
+from .._typing import Literal
 
 ButtonVariant = Literal["default", "primary", "success", "warning", "error"]
 _VALID_BUTTON_VARIANTS = {"default", "primary", "success", "warning", "error"}
@@ -173,12 +168,12 @@ class Button(Static, can_focus=True):
         """Create a Button widget.
 
         Args:
-            label (str): The text that appears within the button.
-            disabled (bool): Whether the button is disabled or not.
-            variant (ButtonVariant): The variant of the button.
-            name: The name of the button.
-            id: The ID of the button in the DOM.
-            classes: The CSS classes of the button.
+            label (str, optional): The text that appears within the button.
+            disabled (bool, optional): Whether the button is disabled or not.
+            variant (ButtonVariant, optional): The variant of the button.
+            name (str | None, optional): The name of the button.
+            id (str | None, optional): The ID of the button in the DOM.
+            classes (str | None, optional): The CSS classes of the button.
         """
         super().__init__(name=name, id=id, classes=classes)
 
@@ -191,7 +186,7 @@ class Button(Static, can_focus=True):
         if disabled:
             self.add_class("-disabled")
 
-        self.variant = variant
+        self.variant = self.validate_variant(variant)
 
     label: Reactive[RenderableType] = Reactive("")
     variant = Reactive.init("default")
@@ -272,11 +267,11 @@ class Button(Static, can_focus=True):
         """Utility constructor for creating a success Button variant.
 
         Args:
-            label (str): The text that appears within the button.
-            disabled (bool): Whether the button is disabled or not.
-            name: The name of the button.
-            id: The ID of the button in the DOM.
-            classes: The CSS classes of the button.
+            label (str, optional): The text that appears within the button.
+            disabled (bool, optional): Whether the button is disabled or not.
+            name (str | None, optional): The name of the button.
+            id (str | None, optional): The ID of the button in the DOM.
+            classes(str | None, optional): The CSS classes of the button.
 
         Returns:
             Button: A Button widget of the 'success' variant.
@@ -303,11 +298,11 @@ class Button(Static, can_focus=True):
         """Utility constructor for creating a warning Button variant.
 
         Args:
-            label (str): The text that appears within the button.
-            disabled (bool): Whether the button is disabled or not.
-            name: The name of the button.
-            id: The ID of the button in the DOM.
-            classes: The CSS classes of the button.
+            label (str, optional): The text that appears within the button.
+            disabled (bool, optional): Whether the button is disabled or not.
+            name (str | None, optional): The name of the button.
+            id (str | None, optional): The ID of the button in the DOM.
+            classes (str | None, optional): The CSS classes of the button.
 
         Returns:
             Button: A Button widget of the 'warning' variant.
@@ -334,11 +329,11 @@ class Button(Static, can_focus=True):
         """Utility constructor for creating an error Button variant.
 
         Args:
-            label (str): The text that appears within the button.
-            disabled (bool): Whether the button is disabled or not.
-            name: The name of the button.
-            id: The ID of the button in the DOM.
-            classes: The CSS classes of the button.
+            label (str, optional): The text that appears within the button.
+            disabled (bool, optional): Whether the button is disabled or not.
+            name (str | None, optional): The name of the button.
+            id (str | None, optional): The ID of the button in the DOM.
+            classes (str | None, optional): The CSS classes of the button.
 
         Returns:
             Button: A Button widget of the 'error' variant.
